@@ -3,7 +3,6 @@ use strict;
 use Carp ();
 use Scalar::Util;
 use IO::Handle;
-use Try::Tiny;
 
 sub TRUE()  { 1==1 }
 sub FALSE() { !TRUE }
@@ -117,13 +116,11 @@ sub load_psgi {
 sub run_app($$) {
     my($app, $env) = @_;
 
-    return try {
-        $app->($env);
-    } catch {
+    return eval { $app->($env) } || do {
         my $body = "Internal Server Error";
-        $env->{'psgi.errors'}->print($_);
-        return [ 500, [ 'Content-Type' => 'text/plain', 'Content-Length' => length($body) ], [ $body ] ];
-    }
+        $env->{'psgi.errors'}->print($@);
+        [ 500, [ 'Content-Type' => 'text/plain', 'Content-Length' => length($body) ], [ $body ] ];
+    };
 }
 
 sub headers {
